#
# Copyright (c) 2019, Grigoriy Kramarenko
# All rights reserved.
# This file is distributed under the BSD 3-Clause License.
#
from django.urls import path
from textrank.views import (
    index, groups, keywords, rank, weightform, samples, topics
)

app_name = 'textrank'


urlpatterns = [
    path('', index, name='index'),
    path('groups/', groups, name='groups'),
    path('groups/<int:id>/', groups, name='group'),
    path('keywords/', keywords, name='keywords'),
    path('keywords/<int:id>/', keywords, name='keyword'),
    path('rank/', rank, name='rank'),
    path('rank/<str:format>/', rank, name='rank'),
    path('weightform/', weightform, name='add_weight'),
    path('weightform/<int:id>/', weightform, name='change_weight'),
    path('samples/', samples, name='samples'),
    path('samples/<int:id>/', samples, name='sample'),
    path('topics/', topics, name='topics'),
    path('topics/<int:id>/', topics, name='topic'),
]



# Generated by Django 3.2.7 on 2021-09-24 08:05

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.crypto
import textrank.validators


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('textrank', '0004_alter_group_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Topic',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='создан')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='обновлён')),
                ('name', models.CharField(error_messages={'unique': 'Такое название уже существует.'}, help_text='Обязательно. 150 символов и менее.', max_length=150, unique=True, verbose_name='название')),
                ('description', models.TextField(blank=True, db_index=True, verbose_name='описание')),
                ('code', models.CharField(default=django.utils.crypto.get_random_string, error_messages={'unique': 'Такой код уже существует.'}, help_text='Обязательно. 32 символа и менее, уникальное значение.', max_length=32, unique=True, verbose_name='код раздела')),
                ('is_active', models.BooleanField(db_index=True, default=True, help_text='Отключите, если раздел не используется для анализа.', verbose_name='активный')),
            ],
            options={
                'verbose_name': 'раздел ранжирования',
                'verbose_name_plural': 'разделы ранжирования',
                'ordering': ('name',),
            },
        ),
        migrations.AddField(
            model_name='group',
            name='topic',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='textrank.topic', verbose_name='раздел'),
        ),
        migrations.AddField(
            model_name='keyword',
            name='topic',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='textrank.topic', verbose_name='раздел'),
        ),
        migrations.AlterField(
            model_name='group',
            name='code',
            field=models.CharField(default=django.utils.crypto.get_random_string, error_messages={'unique': 'Такой код уже существует.'}, help_text='Обязательно. 32 символа и менее, уникальное значение.', max_length=32, verbose_name='код группы'),
        ),
        migrations.AlterField(
            model_name='group',
            name='name',
            field=models.CharField(error_messages={'unique': 'Такое название уже существует.'}, help_text='Обязательно. 150 символов и менее.', max_length=150, verbose_name='название'),
        ),
        migrations.AlterField(
            model_name='keyword',
            name='word',
            field=models.CharField(error_messages={'unique': 'Такое слово (сочетание) уже существует.'}, help_text='Обязательно. 100 символов и менее. Значение может содержать только буквы в нижнем регистре, цифры, символы дефиса, подчёркивания или знак +, а также начинаться и заканчиваться словом. Варианты: Ключевое слово (word1), цепочка слов (word1_word2_word3) или покрытие словами (word1+word2+word3).', max_length=100, validators=[textrank.validators.UnicodeKeywordValidator()], verbose_name='слово'),
        ),
        migrations.AddConstraint(
            model_name='group',
            constraint=models.UniqueConstraint(condition=models.Q(('topic__isnull', True)), fields=('name',), name='group_name_default_unique'),
        ),
        migrations.AddConstraint(
            model_name='group',
            constraint=models.UniqueConstraint(condition=models.Q(('topic__isnull', False)), fields=('name', 'topic'), name='group_name_topic_unique'),
        ),
        migrations.AddConstraint(
            model_name='group',
            constraint=models.UniqueConstraint(condition=models.Q(('topic__isnull', True)), fields=('code',), name='group_code_default_unique'),
        ),
        migrations.AddConstraint(
            model_name='group',
            constraint=models.UniqueConstraint(condition=models.Q(('topic__isnull', False)), fields=('code', 'topic'), name='group_code_topic_unique'),
        ),
        migrations.AddConstraint(
            model_name='keyword',
            constraint=models.UniqueConstraint(condition=models.Q(('topic__isnull', True)), fields=('word',), name='keyword_default_unique'),
        ),
        migrations.AddConstraint(
            model_name='keyword',
            constraint=models.UniqueConstraint(condition=models.Q(('topic__isnull', False)), fields=('word', 'topic'), name='keyword_topic_unique'),
        ),
        migrations.AddField(
            model_name='topic',
            name='last_editor',
            field=models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='последний редактор'),
        ),
    ]



#
# Copyright (c) 2019, Grigoriy Kramarenko
# All rights reserved.
# This file is distributed under the BSD 3-Clause License.
#
from django.apps import AppConfig
from djangokit.utils.apps import DependencesMixin


class DefaultConfig(DependencesMixin, AppConfig):
    default_auto_field = 'django.db.models.AutoField'
    name = 'textrank'
    dependences = [
        'django.contrib.auth',
        'django.contrib.sessions',
        'djangokit',
    ]



#
# Copyright (c) 2019, Grigoriy Kramarenko
# All rights reserved.
# This file is distributed under the BSD 3-Clause License.
#
from json import loads

from django.http import QueryDict, HttpResponseForbidden, JsonResponse
from django.shortcuts import get_object_or_404, resolve_url
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views.decorators.csrf import csrf_exempt

from djangokit.views import controller

from textrank import __version__
from textrank.conf import LOGIN_URL, LOGOUT_URL, DEMO_MODE, JSON_DUMPS_PARAMS
from textrank.decorators import user_access_required, api_access_required


def get_default_context():
    ctx = {
        'VERSION': __version__,
        'LOGIN_URL': resolve_url(LOGIN_URL),
        'LOGOUT_URL': resolve_url(LOGOUT_URL),
        'DEMO_MODE': DEMO_MODE,
    }
    return ctx


def parse_params(request):
    if request.content_type == 'application/json':
        return loads(request.body.decode('utf-8'))
    method = request.method
    if method == 'GET':
        return request.GET
    elif method == 'POST':
        return request.POST
    return QueryDict(request.body)


@method_decorator(user_access_required, name='dispatch')
class UserAccessMixin:
    """Mixing for user access."""

    def dispatch(self, request, *args, **kwargs):
        if DEMO_MODE and request.method != 'GET':
            # В демо-режиме не позволяем анонимам изменять данные.
            if not request.user.is_authenticated:
                return HttpResponseForbidden(_(
                    'Авторизируйтесь, чтобы иметь возможность редактировать '
                    'данные.'
                ))
        return super(UserAccessMixin, self).dispatch(request, *args, **kwargs)


@method_decorator(csrf_exempt, name='dispatch')
@method_decorator(api_access_required, name='dispatch')
class ApiAccessMixin:
    """Mixing for API access."""
    def dispatch(self, *args, **kwargs):
        return super(ApiAccessMixin, self).dispatch(*args, **kwargs)


class BaseModelView(controller.ControlModelView):

    def get_default_context(self, *args, **kwargs):
        return get_default_context()

    def render_to_json_response(self, ctx):
        form = ctx['form']
        if form.is_bound and form._errors:
            data = {'errors': form.errors.get_json_data()}
            return JsonResponse(data, json_dumps_params=JSON_DUMPS_PARAMS,
                                status=400)

        instance = ctx['instance']
        if instance:
            data = self.ctrl.serialize(instance)
        else:
            data = {
                'page': self.ctrl.serialize_page(ctx['page'], None),
                'orders': ctx['orders'],
                'filters': ctx['filters'],
            }
        return JsonResponse(data, json_dumps_params=JSON_DUMPS_PARAMS)

    def render_to_response(self, ctx):
        if self.request.GET.get('format') == 'json':
            return self.render_to_json_response(ctx)
        return super().render_to_response(ctx)

    def delete(self, request, id=None):
        if id:
            instance = get_object_or_404(self.model, id=id)
            instance.delete()
        else:
            self.model.objects.all().delete()
        return JsonResponse({'success': True, 'id': id})



#
# Copyright (c) 2019, Grigoriy Kramarenko
# All rights reserved.
# This file is distributed under the BSD 3-Clause License.
#
from django.db.models import Count
from django.http.response import (
    JsonResponse, HttpResponse, HttpResponseBadRequest, Http404)
from django.shortcuts import redirect, get_object_or_404
from django.utils.translation import gettext_lazy as _
from django.views.generic import TemplateView, View

from djangokit.views.controller import Controller

from textrank.conf import JSON_DUMPS_PARAMS
from textrank.forms import (
    GroupForm, KeywordForm, RankForm, RegForm, WeightForm, SampleForm, TopicForm
)
from textrank.models import Group, Keyword, Weight, Sample, Topic
from textrank.views.base import (
    get_default_context, UserAccessMixin, ApiAccessMixin, BaseModelView,
)


class IndexView(UserAccessMixin, TemplateView):
    template_name = 'textrank/index.html'

    @property
    def extra_context(self):
        ctx = get_default_context()
        # Группы
        groups = Group.objects.order_by('name')
        ctx['all_groups'] = groups
        ctx['last_groups'] = groups.order_by('-updated', 'name')[:3]
        ctx['active_groups'] = groups.filter(is_active=True)
        ctx['notactive_groups'] = groups.filter(is_active=False)
        # Ключевые слова
        keywords = Keyword.objects.order_by('word')
        ctx['all_keywords'] = keywords
        ctx['last_keywords'] = keywords.order_by('-updated', 'word')[:5]
        ctx['active_keywords'] = keywords.filter(is_active=True)
        ctx['notactive_keywords'] = keywords.filter(is_active=False)
        # Последние досбавленные веса
        weights = Weight.objects.select_related('group', 'keyword')
        ctx['last_weights'] = weights.order_by('-updated', 'keyword__word')[:8]
        return ctx

    def get(self, request):
        """Отображает главную страницу настройки анализа."""
        ctx = self.get_context_data()
        ctx['form'] = RegForm(initial=request.GET)
        return self.render_to_response(ctx)

    def post(self, request):
        """Отображает главную страницу настройки анализа."""
        ctx = self.get_context_data()
        ctx['form'] = form = RegForm(request.POST)
        if form.is_valid():
            ctx['result'] = form.save(request.user)
            cdata = form.cleaned_data
            ctx['form'] = RegForm(initial={
                'groups': ','.join(cdata['groups']),
                'weight': cdata['weight'],
            })
        return self.render_to_response(ctx)


def group_serializer(group):
    data = {
        'id': group.id,
        'created': group.created,
        'updated': group.updated,
        'name': group.name,
        'code': group.code,
        'is_active': group.is_active,
        'topic': None,
        'last_editor': None,
    }
    last_editor = group.last_editor
    if last_editor:
        data['last_editor'] = [last_editor.id, str(last_editor)]
    topic = group.last_editor
    if topic:
        data['topic'] = [topic.id, str(topic)]
    return data


class GroupController(Controller):
    select_related = ('last_editor', 'topic')
    filtering_fields = [f.name for f in Group._meta.fields] + [
        'keywords_count',
    ]
    ordering_fields = [
        f.name for f in Group._meta.fields if not f.related_model
    ] + ['keywords_count']

    def get_queryset(self):
        qs = self.model.objects.all()
        qs = qs.annotate(keywords_count=Count('weights'))
        return qs


class GroupsView(ApiAccessMixin, BaseModelView):
    template_name = 'textrank/groups.html'
    ctrl = GroupController(Group, serializer=group_serializer)
    model = Group
    form = GroupForm


def keyword_serializer(keyword):
    data = {
        'id': keyword.id,
        'created': keyword.created,
        'updated': keyword.updated,
        'word': keyword.word,
        'is_active': keyword.is_active,
        'last_editor': None,
    }
    last_editor = keyword.last_editor
    if last_editor:
        data['last_editor'] = [last_editor.id, str(last_editor)]
    if keyword.id:
        qs = Weight.object.select_related('group', 'last_editor')
        qs = qs.filter(keyword=keyword)

        def serialize_weight(w):
            g = w.group
            dic = {
                'group': [g.id, g.name],
                'value': w.value,
            }
            le = w.last_editor
            if le:
                dic['last_editor'] = [le.id, str(le)]
            return dic

        data['weights'] = [serialize_weight(w) for w in qs]

    return data


class KeywordsView(UserAccessMixin, BaseModelView):
    template_name = 'textrank/keywords.html'
    ctrl = Controller(Keyword, serializer=keyword_serializer)
    ctrl.select_related = ('last_editor',)
    ctrl.prefetch_related = ('weights',)
    model = Keyword
    form = KeywordForm

    def get_default_context(self):
        ctx = super().get_default_context()
        ctx['groups'] = Group.objects.all()
        return ctx


class RankView(ApiAccessMixin, TemplateView):
    template_name = 'textrank/rank.html'

    def get(self, request, format='html'):
        """Отображает форму на странице."""

        if format not in ('html', 'text', 'json'):
            raise Http404(_('Неверный формат.'))

        ctx = get_default_context()
        ctx['format'] = format
        ctx['form'] = RankForm(initial=request.GET)
        return self.render_to_response(ctx)

    def post(self, request, format='html'):
        """Ранжирует текст и выдаёт результат формой, текстом или в JSON."""

        if format not in ('html', 'text', 'json'):
            raise Http404(_('Неверный формат.'))

        params = request.POST
        form = RankForm(params)
        # HTML
        if format == 'html':
            ctx = get_default_context()
            ctx['form'] = form
            ctx['format'] = format
            if form.is_valid():
                ctx['result'] = form.result_as_data()
            return self.render_to_response(ctx)
        # Plain text
        if format == 'text':
            if form.is_valid():
                return HttpResponse(form.result_as_text())
            return HttpResponseBadRequest(str(form.errors))
        # JSON
        if form.is_valid():
            data = form.result_as_data()
        else:
            data = {
                'errors': form.errors.get_json_data(),
            }
        return JsonResponse(data, json_dumps_params=JSON_DUMPS_PARAMS)


class WeightFormView(UserAccessMixin, View):
    """Представление для добавления или изменения веса."""

    def post(self, request, id=None):
        if id:
            weight = get_object_or_404(Weight, id=id)
        else:
            weight = Weight()
        form = WeightForm(request.POST, instance=weight, request=request)
        if form.is_valid():
            form.save()
            if 'next' in request.POST:
                return redirect(request.POST['next'])
            data = {'success': True}
        else:
            data = {'errors': form.errors.get_json_data(escape_html=True)}
        return JsonResponse(data, json_dumps_params=JSON_DUMPS_PARAMS)


def sample_serializer(sample):
    group = sample.group
    data = {
        'id': sample.id,
        'created': sample.created,
        'updated': sample.updated,
        'message': sample.message,
        'checksum': sample.checksum,
        'group': [group.id, str(group)],
        'last_editor': None,
    }
    last_editor = sample.last_editor
    if last_editor:
        data['last_editor'] = [last_editor.id, str(last_editor)]
    return data


class SampleController(Controller):
    select_related = ('group', 'last_editor',)
    search_fields = ['message']


class SamplesView(ApiAccessMixin, BaseModelView):
    template_name = 'textrank/samples.html'
    ctrl = SampleController(Sample, serializer=sample_serializer)
    model = Sample
    form = SampleForm

    def get_default_context(self):
        ctx = super().get_default_context()
        ctx['groups'] = Group.objects.all()
        return ctx


def topic_serializer(topic):
    data = {
        'id': topic.id,
        'created': topic.created,
        'updated': topic.updated,
        'name': topic.name,
        'description': topic.description,
        'code': topic.code,
        'is_active': topic.is_active,
        'last_editor': None,
    }
    last_editor = topic.last_editor
    if last_editor:
        data['last_editor'] = [last_editor.id, str(last_editor)]
    return data


class TopicController(Controller):
    select_related = ('last_editor',)
    filtering_fields = [f.name for f in Group._meta.fields]
    ordering_fields = [
        f.name for f in Group._meta.fields if not f.related_model
    ]


class TopicsView(ApiAccessMixin, BaseModelView):
    template_name = 'textrank/topics.html'
    ctrl = TopicController(Topic, serializer=topic_serializer)
    model = Topic
    form = TopicForm



# Generated by Django 3.2.7 on 2021-09-16 23:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('textrank', '0003_group_description'),
    ]

    operations = [
        migrations.AlterField(
            model_name='group',
            name='name',
            field=models.CharField(error_messages={'unique': 'Такое название уже существует.'}, help_text='Обязательно. 150 символов и менее.', max_length=150, unique=True, verbose_name='название'),
        ),
    ]



from textrank.views.pages import (
    IndexView, GroupsView, KeywordsView, RankView, WeightFormView, SamplesView,
    TopicsView
)

index = IndexView.as_view()
groups = GroupsView.as_view()
keywords = KeywordsView.as_view()
rank = RankView.as_view()
weightform = WeightFormView.as_view()
samples = SamplesView.as_view()
topics = TopicsView.as_view()



#
# Copyright (c) 2019, Grigoriy Kramarenko
# All rights reserved.
# This file is distributed under the BSD 3-Clause License.
#
from django.contrib import admin
from textrank import models


class TopicAdmin(admin.ModelAdmin):
    # save_as = True
    date_hierarchy = 'updated'
    list_display = (
        'name', 'code', 'is_active', 'updated', 'last_editor', 'id',
    )
    list_filter = ['is_active', 'updated']
    search_fields = ['name']


class WeightInline(admin.StackedInline):
    model = models.Weight
    extra = 0


class GroupAdmin(admin.ModelAdmin):
    # save_as = True
    date_hierarchy = 'updated'
    list_display = (
        'name', 'is_active', 'updated', 'last_editor', 'id',
    )
    list_filter = ['is_active', 'updated']
    search_fields = ['name']


class KeywordAdmin(admin.ModelAdmin):
    save_as = True
    date_hierarchy = 'updated'
    list_display = (
        'word', 'is_active', 'updated', 'last_editor', 'id',
    )
    list_filter = ['is_active', 'updated']
    search_fields = ['word']
    inlines = [WeightInline]


class WeightAdmin(admin.ModelAdmin):
    date_hierarchy = 'updated'
    list_display = (
        'keyword', 'value', 'group', 'updated', 'last_editor', 'id',
    )
    list_filter = ['group__is_active', 'keyword__is_active', 'updated']
    search_fields = ['group__name', 'keyword__word']

    def get_queryset(self, *args, **kwargs):
        qs = super().get_queryset(*args, **kwargs)
        return qs.select_related('group', 'keyword')


register = admin.site.register
register(models.Topic, TopicAdmin)
register(models.Group, GroupAdmin)
register(models.Keyword, KeywordAdmin)
register(models.Weight, WeightAdmin)



#
# Copyright (c) 2019, Grigoriy Kramarenko
# All rights reserved.
# This file is distributed under the BSD 3-Clause License.
#
from collections import OrderedDict
from hashlib import md5
from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.timezone import now
from django.utils.crypto import get_random_string
from django.utils.translation import gettext_lazy as _

from textrank.validators import keyword_validator


class Topic(models.Model):
    """Раздел для группировки ранжирования."""
    created = models.DateTimeField(_('создан'), auto_now_add=True)
    updated = models.DateTimeField(_('обновлён'), auto_now=True)
    name = models.CharField(
        _('название'), max_length=150, unique=True,
        help_text=_('Обязательно. 150 символов и менее.'),
        error_messages={
            'unique': _('Такое название уже существует.'),
        },
    )
    description = models.TextField(
        _('описание'), blank=True, db_index=True,
    )
    code = models.CharField(
        _('код раздела'), max_length=32, default=get_random_string, unique=True,
        help_text=_('Обязательно. 32 символа и менее, уникальное значение.'),
        error_messages={
            'unique': _('Такой код уже существует.'),
        },
    )
    is_active = models.BooleanField(
        _('активный'), default=True, db_index=True, help_text=_(
            'Отключите, если раздел не используется для анализа.'
        )
    )
    # Пользователь, который последним изменил объект.
    last_editor = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True, blank=True,
        editable=False,
        verbose_name=_('последний редактор'),
        on_delete=models.PROTECT,
    )

    class Meta:
        verbose_name = _('раздел ранжирования')
        verbose_name_plural = _('разделы ранжирования')
        ordering = ('name',)

    def __str__(self):
        return self.name

    def delete(self, **kwargs):
        # Удалять объекты в Django запрещено. Только из базы данных.
        self.is_active = False
        self.save(**kwargs)

    def get_description(self):
        return self.description.replace('\r\n', ' ').replace('\n', ' ')


class Group(models.Model):
    """Модель группы ключевых слов."""
    created = models.DateTimeField(_('создана'), auto_now_add=True)
    updated = models.DateTimeField(_('обновлена'), auto_now=True)

    name = models.CharField(
        _('название'), max_length=150,
        help_text=_('Обязательно. 150 символов и менее.'),
        error_messages={
            'unique': _('Такое название уже существует.'),
        },
    )
    description = models.TextField(
        _('описание'), blank=True, db_index=True,
    )
    code = models.CharField(
        _('код группы'), max_length=32, default=get_random_string,
        help_text=_('Обязательно. 32 символа и менее, уникальное значение.'),
        error_messages={
            'unique': _('Такой код уже существует.'),
        },
    )
    is_active = models.BooleanField(
        _('активная'), default=True, db_index=True, help_text=_(
            'Отключите, если группа не используется для анализа.'
        )
    )
    topic = models.ForeignKey(
        Topic,
        null=True, blank=True,
        verbose_name=_('раздел'),
        on_delete=models.PROTECT,
    )
    # Пользователь, который последним изменил объект.
    last_editor = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True, blank=True,
        editable=False,
        verbose_name=_('последний редактор'),
        on_delete=models.PROTECT,
    )

    class Meta:
        verbose_name = _('группа ключевых слов')
        verbose_name_plural = _('группы ключевых слов')
        ordering = ('name',)
        constraints = [
            models.UniqueConstraint(
                fields=['name'], name='group_name_default_unique',
                condition=models.Q(topic__isnull=True),
            ),
            models.UniqueConstraint(
                fields=['name', 'topic'], name='group_name_topic_unique',
                condition=models.Q(topic__isnull=False),
            ),
            models.UniqueConstraint(
                fields=['code'], name='group_code_default_unique',
                condition=models.Q(topic__isnull=True),
            ),
            models.UniqueConstraint(
                fields=['code', 'topic'], name='group_code_topic_unique',
                condition=models.Q(topic__isnull=False),
            ),
        ]

    def __str__(self):
        return self.name

    def save(self, **kwargs):
        super().save(**kwargs)
        # Обновляем все веса, чтобы анализатор перезагрузил данные.
        Weight.objects.filter(group=self).update(updated=now())

    def delete(self, **kwargs):
        # Удалять объекты в Django запрещено. Только из базы данных.
        self.is_active = False
        self.save(**kwargs)

    def get_all_keywords(self):
        return self.keyword_set.all()

    def get_active_keywords(self):
        return self.get_all_keywords().filter(is_active=True)

    def get_inactive_keywords(self):
        return self.get_all_keywords().filter(is_active=False)

    def get_all_weights(self):
        qs = self.weights.select_related('group', 'keyword')
        qs = qs.order_by('keyword__word')
        return qs

    def get_description(self):
        return self.description.replace('\r\n', ' ').replace('\n', ' ')


class Keyword(models.Model):
    """Модель ключевого слова или словосочетания."""

    created = models.DateTimeField(_('создано'), auto_now_add=True)
    updated = models.DateTimeField(_('обновлено'), auto_now=True)
    word = models.CharField(
        _('слово'), max_length=100,
        help_text=_(
            'Обязательно. 100 символов и менее. '
            'Значение может содержать только буквы в нижнем регистре, '
            'цифры, символы дефиса, подчёркивания или знак +, а также '
            'начинаться и заканчиваться словом. Варианты: '
            'Ключевое слово (word1), цепочка слов (word1_word2_word3) '
            'или покрытие словами (word1+word2+word3).'
        ),
        validators=[keyword_validator],
        error_messages={
            'unique': _('Такое слово (сочетание) уже существует.'),
        },
    )
    is_active = models.BooleanField(
        _('активное'), default=True, db_index=True, help_text=_(
            'Отключите, если слово не используется для анализа.'
        )
    )
    topic = models.ForeignKey(
        Topic,
        null=True, blank=True,
        verbose_name=_('раздел'),
        on_delete=models.PROTECT,
    )
    # Пользователь, который последним изменил объект.
    last_editor = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True, blank=True,
        editable=False,
        verbose_name=_('последний редактор'),
        on_delete=models.PROTECT,
    )
    groups = models.ManyToManyField(
        Group, through='Weight',
        editable=False,
        verbose_name=_('группы')
    )

    class Meta:
        verbose_name = _('ключевое слово')
        verbose_name_plural = _('ключевые слова')
        ordering = ('word',)
        constraints = [
            models.UniqueConstraint(
                fields=['word'], name='keyword_default_unique',
                condition=models.Q(topic__isnull=True),
            ),
            models.UniqueConstraint(
                fields=['word', 'topic'], name='keyword_topic_unique',
                condition=models.Q(topic__isnull=False),
            ),
        ]

    def __str__(self):
        return self.word

    @staticmethod
    def validate_word(word, raise_error=False):
        try:
            keyword_validator(word)
        except ValidationError as e:
            if raise_error:
                raise e
            return False
        return True

    @property
    def is_chain(self):
        return '_' in self.word

    @property
    def is_cover(self):
        return '+' in self.word

    @property
    def chainwords(self):
        w = self.word
        if '_' in w:
            words = [x for x in w.split('_') if x]
            if len(words) >= 2:
                return words
        return []

    @property
    def coverwords(self):
        w = self.word
        if '+' in w:
            words = [x for x in w.split('+') if x]
            if len(words) >= 2:
                return words
        return []

    def get_all_groups(self):
        return self.groups.all()

    def get_active_groups(self):
        return self.get_all_groups().filter(is_active=True)

    def get_inactive_groups(self):
        return self.get_all_groups().filter(is_active=False)

    def get_weight_groups(self):
        groups = OrderedDict([(g.id, [g, None]) for g in Group.objects.all()])
        for w in self.weights.select_related('group', 'keyword'):
            groups[w.group_id][1] = w
        return groups

    def save(self, **kwargs):
        self.validate_word(self.word, True)
        super().save(**kwargs)
        # Обновляем все веса, чтобы анализатор перезагрузил данные.
        Weight.objects.filter(keyword=self).update(updated=now())

    def delete(self, **kwargs):
        # Удалять объекты в Django запрещено. Только из базы данных.
        self.is_active = False
        self.save(**kwargs)


class Weight(models.Model):
    """Модель веса ключевого слова для группы."""
    created = models.DateTimeField(_('создан'), auto_now_add=True)
    updated = models.DateTimeField(_('обновлён'), auto_now=True, db_index=True)
    group = models.ForeignKey(
        Group,
        verbose_name=_('группа'),
        on_delete=models.PROTECT,
        related_name='weights',
    )
    keyword = models.ForeignKey(
        Keyword,
        verbose_name=_('ключевое слово'),
        on_delete=models.PROTECT,
        related_name='weights',
    )
    value = models.PositiveSmallIntegerField(_('значение веса'), default=1)
    # Пользователь, который последним изменил объект.
    last_editor = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True, blank=True,
        editable=False,
        verbose_name=_('последний редактор'),
        on_delete=models.PROTECT,
    )

    class Meta:
        verbose_name = _('вес ключевого слова')
        verbose_name_plural = _('веса ключевых слов')
        unique_together = ('group', 'keyword')
        ordering = ('-updated',)

    def __str__(self):
        return '%s = %s' % (self.keyword, self.value)

    @property
    def is_active(self):
        return bool(
            self.value and self.group.is_active and self.keyword.is_active)


class Sample(models.Model):
    """
    Модель образца текста для однозначного сопоставления с группой.
    Образцы собираются для обучения моделей искусственного интеллекта.
    """
    created = models.DateTimeField(_('создан'), auto_now_add=True)
    updated = models.DateTimeField(_('обновлён'), auto_now=True, db_index=True)
    checksum = models.CharField(
        _('контрольная сумма'), max_length=32, blank=True, editable=False)
    group = models.ForeignKey(
        Group,
        verbose_name=_('группа'),
        on_delete=models.PROTECT,
        related_name='samples',
    )
    message = models.TextField(_('текст сообщения'))
    # Пользователь, который последним изменил объект.
    last_editor = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True, blank=True,
        editable=False,
        verbose_name=_('последний редактор'),
        on_delete=models.PROTECT,
    )

    class Meta:
        verbose_name = _('образец')
        verbose_name_plural = _('образцы')
        unique_together = ('group', 'checksum')
        ordering = ('-updated',)

    def __str__(self):
        return self.message

    @staticmethod
    def get_checksum(message):
        return md5(message.encode('utf-8')).hexdigest()

    def save(self, **kwargs):
        self.checksum = self.get_checksum(self.message)
        super().save(**kwargs)



# Generated by Django 3.0 on 2019-12-20 11:33

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.crypto
import textrank.validators


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='создана')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='обновлена')),
                ('name', models.CharField(error_messages={'unique': 'Такое название уже существует.'}, help_text='Обязательно. 50 символов и менее.', max_length=50, unique=True, verbose_name='название')),
                ('code', models.CharField(default=django.utils.crypto.get_random_string, error_messages={'unique': 'Такой код уже существует.'}, help_text='Обязательно. 32 символа и менее, уникальное значение.', max_length=32, unique=True, verbose_name='код группы')),
                ('is_active', models.BooleanField(db_index=True, default=True, help_text='Отключите, если группа не используется для анализа.', verbose_name='активная')),
                ('last_editor', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='последний редактор')),
            ],
            options={
                'verbose_name': 'группа ключевых слов',
                'verbose_name_plural': 'группы ключевых слов',
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='Keyword',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='создано')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='обновлено')),
                ('word', models.CharField(error_messages={'unique': 'Такое слово (сочетание) уже существует.'}, help_text='Обязательно. 100 символов и менее. Значение может содержать только буквы в нижнем регистре, цифры, символы дефиса, подчёркивания или знак +, а также начинаться и заканчиваться словом. Варианты: Ключевое слово (word1), цепочка слов (word1_word2_word3) или покрытие словами (word1+word2+word3).', max_length=100, unique=True, validators=[textrank.validators.UnicodeKeywordValidator()], verbose_name='слово')),
                ('is_active', models.BooleanField(db_index=True, default=True, help_text='Отключите, если слово не используется для анализа.', verbose_name='активное')),
            ],
            options={
                'verbose_name': 'ключевое слово',
                'verbose_name_plural': 'ключевые слова',
                'ordering': ('word',),
            },
        ),
        migrations.CreateModel(
            name='Weight',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='создан')),
                ('updated', models.DateTimeField(auto_now=True, db_index=True, verbose_name='обновлён')),
                ('value', models.PositiveSmallIntegerField(default=1, verbose_name='значение веса')),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='weights', to='textrank.Group', verbose_name='группа')),
                ('keyword', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='weights', to='textrank.Keyword', verbose_name='ключевое слово')),
                ('last_editor', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='последний редактор')),
            ],
            options={
                'verbose_name': 'вес ключевого слова',
                'verbose_name_plural': 'веса ключевых слов',
                'ordering': ('-updated',),
                'unique_together': {('group', 'keyword')},
            },
        ),
        migrations.AddField(
            model_name='keyword',
            name='groups',
            field=models.ManyToManyField(editable=False, through='textrank.Weight', to='textrank.Group', verbose_name='группы'),
        ),
        migrations.AddField(
            model_name='keyword',
            name='last_editor',
            field=models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='последний редактор'),
        ),
    ]



# Generated by Django 3.0.4 on 2021-09-15 05:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('textrank', '0002_sample'),
    ]

    operations = [
        migrations.AddField(
            model_name='group',
            name='description',
            field=models.TextField(blank=True, db_index=True, verbose_name='описание'),
        ),
    ]
